/*
 *  Olvid for iOS
 *  Copyright © 2019-2022 Olvid SAS
 *
 *  This file is part of Olvid for iOS.
 *
 *  Olvid is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License, version 3,
 *  as published by the Free Software Foundation.
 *
 *  Olvid is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with Olvid.  If not, see <https://www.gnu.org/licenses/>.
 */

import UIKit
import SwiftUI


protocol OlvidMenuProvider: UIViewController {
    
    @available(iOS 13, *)
    func provideMenu() -> UIMenu
    
    @available(iOS, introduced: 13, deprecated: 14, message: "Use provideMenu() instead")
    func provideAlertActions() -> [UIAlertAction]

}


extension UIViewController {
    
    @available(iOS 13, *)
    func getFirstMenuAvailable() -> UIMenu? {
        assert(Thread.isMainThread)
        var currentViewController: UIViewController? = self
        while let candidate = currentViewController {
            if let parentMenuProvider = candidate as? OlvidMenuProvider {
                return parentMenuProvider.provideMenu()
            }
            currentViewController = currentViewController?.parent
        }
        return nil
    }

    
    @available(iOS, introduced: 13, deprecated: 14, message: "Use getFirstParentMenuAvailable() instead")
    func getFirstAlertActionsAvailable() -> [UIAlertAction] {
        assert(Thread.isMainThread)
        var currentViewController: UIViewController? = self
        while let candidate = currentViewController {
            if let parentMenuProvider = candidate as? OlvidMenuProvider {
                return parentMenuProvider.provideAlertActions()
            }
            currentViewController = currentViewController?.parent
        }
        return []
    }

}
